// **************************************************************************
//    Bibliothèque sommaire pour la manipulation d'impulsions femtosecondes
// **************************************************************************

clear();   // Pas indispensable met permet d'éviter le message :
           //         Warning : redefining function
           // à chaque fois que l'on recharge le programme.

// Modification des limites de l'axe des abscisses d'un graphe
// Doit être appelé juste après plot
// Directement disponible dans Matlab
function xlim(x)
  ax = gca();
  ax.data_bounds(1,1) = x(1);
  ax.data_bounds(2,1) = x(2);
endfunction;

// Modification des limites de l'axe des ordonnées d'un graphe
// Doit être appelé juste après plot
// Directement disponible dans Matlab
function ylim(y)
  ax = gca();
  ax.data_bounds(1,2) = y(1);
  ax.data_bounds(2,2) = y(2);
endfunction;

// Phase (en radians) d'un nombre complexe
// Directement disponible dans Matlab
function result = angle(z)
  z = z + bool2s(z==0); // Pour ajouter 1 et éviter une erreur pour z=0.
  result = atan(imag(z),real(z))
endfunction;

// Supprime par continuité les sauts de 2pi dans un tableau de phases
// Directement disponible dans Matlab
function result = unwrap(phi)
  result = phi - [0 2*%pi*cumsum(floor(.5+diff(phi)/2/%pi))];
endfunction;

// Prend l'inverse d'un vecteur
// Dans Scilab, si x est une matrice n x 1, 1./x donne une matrice 1 x n
// pour que le produit (1./x)*x soit égal à 1 (ce n'est pas le cas dans Matlab)
// Si on veut une division terme à terme, il faut écrire 1 ./x, avec un espace entre
// le 1 et le ./.
// Pour éviter d'oublier l'espace, on peut utiliser inverse
function result = inverse(x)
  result = 1 ./x;
endfunction;

// Prend le carré d'une grandeur (plus rapide que .^2)
function result = sqr(x)
  result = x .* x;
endfunction;

// Fonction de Heaviside
function result = heaviside(x)
  // (x>0) est un tableau de boolean, transformé en 0 ou 1 à l'aide de bool2s
  result = bool2s(x>0);
endfunction;

// Ecart quadratique d'une grandeur x pondérée par l'amplitude de probabilité field
function result = ecartQuad(x,field)
  n2 = sum(sqr(abs(field)));
  xMoy = sum(x.*sqr(abs(field)))/n2;
  x2Moy = sum(x.*x.*sqr(abs(field)))/n2;
  result = sqrt(x2Moy-sqr(xMoy));
endfunction;

// Choix de l'échelle de couleur utilisée pour les représentations en fausses couleurs
// 0 pour niveaux de gris, 1 pour jetcolormap
function setColorMap(i)
  hcf = gcf();
  select i
    case 0 then
      hcf.color_map = graycolormap(256);
    case 1 then
      hcf.color_map = jetcolormap(256);
  end;
endfunction;

// Lecture d'un fichier ascii au format csv (comma separated values)
function result = csvread(fileName)
  fd = mopen(fileName);
  str = mgetl(fd);
  [n dummy] = size(str); // n est le nombre de lignes
  [m dummy] = size(tokens(str(1),",")); // m est le nombre de colonnes
  for i=1:n
    line = tokens(str(i),",");
    for j=1:m
      result(i,j) = sscanf(line(j),"%e");
    end;
  end;
  mclose(fd);
endfunction;

// Transformée de Fourier avec rotation pour centrer la fréquence nulle
function result = ft(data)
  result = fftshift(fft(fftshift(data)));
endfunction;

// Transformée de Fourier inverse avec rotation pour centrer la fréquence nulle
function result = ift(data)
  result = fftshift(ifft(fftshift(data)));
endfunction;

// ftAxis
// Création de deux tableaux (fréquence et temps) calibrés l'un par rapport à l'autre pour
// une FFT. L'espacement entre deux points de fréquence est l'inverse de la largeur totale
// dans l'espace des temps. Inversement, l'espacement entre deux points de temps est l'inverse
// de la largeur totale en fréquence.
// Les axes de fréquence et de temps sont supposés centré sur zéro
//
// nPoints : Nombre de points (de préférence une puissance de 2)
// nuMax   : Valeur maximale de la fréquence (ie moitié de la largeur totale)
// nu : Axe des fréquences. Le point d'indice nPoints/2 vaut toujours zéro
// t  : Axe des temps. Le point d'indice nPoints/2 vaut toujours zéro
function [nu, t] = ftAxis(nPoints, nuMax)
  deltaNu = 2*nuMax/nPoints;
  deltaT = 1/(2*nuMax);
  nu = -nuMax:2*nuMax/nPoints:nuMax-(2*nuMax/nPoints);
  t = -nPoints/2*deltaT:deltaT:(nPoints/2-1)*deltaT;
endfunction;

// Formule de Sellmeier pour le calcul de l'indice de réfraction
// n^2 = 1 + B1/(1-C1/lambda^2) + B2/(1-C2/lambda^2) + B3/(1-C3/lambda^2)
// lambda s'exprime en microns
function result = sellmeier(lambda,b1,b2,b3,c1,c2,c3)
  lm2 = lambda.^-2;
  result = sqrt(1+b1./(1-c1*lm2)+b2./(1-c2*lm2)+b3./(1-c3*lm2));
endfunction;

// Indices ordinaire et extraordinaire de quelques matériaux courants
// La longueur d'onde lambda s'exprime en microns
function [no, ne] = indice(cristal,lambda)
  lambda2 = sqr(lambda);
  select cristal
    case 'BBO' then
      no = sqrt(2.7405+.0184./(lambda2-.0179)-.0155 *lambda2);
      ne = sqrt(2.3730+.0128./(lambda2-.0156)-.0044 *lambda2);
    case 'KDP' then
      no = sqrt(2.259276 + 0.01008956./(lambda2 - 0.012942625) ...
                         + 13.005522*lambda2./(lambda2 - 400));
      ne = sqrt(2.132668 + 0.008637494./(lambda2 - 0.012281043) ...
                         + 3.2279924*lambda2./(lambda2 - 400));
    case 'SiO2' then
      // http://cvilaser.com/Common/PDFs/Dispersion_Equations.pdf
      no = sellmeir(lambda,.6961663,.4079426,.8974794,4.67914826E-3,1.35120631E-2,9.79340025E1);
      ne = no;
    case 'CaF2' then
      // http://www.us.schott.com/lithotec/english/download/caf2_june_2006_final_us.pdf
      no = sellmeier(lambda,.6188140,.4198937,3.426299,2.759866E-3,1.061251E-2,1.068123E3);
//      no = sellmeier(lambda,.567588800,.471091400,3.84847230,.00252642999,.0100783328,1200.55597);
      ne = no;
    else
      no = 1; ne = 1;
  end;
endfunction;

// Indice extraordinaire effectif pour un angle theta donné
function result = neTheta(cristal, theta, lambda)
  [no, ne] = indice(cristal, lambda);
  result = (sqr(cos(theta)./no)+sqr(sin(theta)./ne)).^(-.5);
endfunction;

// Initialisation des paramètres par défaut des représentations graphiques
hda = gda();               // Handle du repère par défaut (default axes)
hda.font_size = 3;         // Augmente la taille de police des axes
hda.title.font_size = 3;   // Augmente la taille du titre
hda.x_label.font_size = 3; // Axe des x
hda.y_label.font_size = 3; // Axe des y
// Pour projection ou figure article, utiliser paramètres ci-dessous
hda.font_size = 4;         // Augmente la taille de police des axes
hda.thickness = 2;         // Epaisseur par défaut des lignes
